searchData={"content_type":"text/markdown","items":[{"doc":"Snap is split into a few key modules:\n\n* `Snap.Cluster` - clusters are wrappers around the Elasticsearch HTTP API.\n  We use these to perform low-level HTTP requests.\n\n* `Snap.Indexes` - a convenience wrapper around the Elasticsearch indexes\n  APIs, allowing the creation, deleting and aliasing of indexes, along with\n  hotswap functionality to bulk load documents into an aliased index,\n  switching to it atomically.\n\n* `Snap.Document` - a convenience wrapper around the Document APIs, allowing\n  the creating, updating and deleting individual documents.\n\n* `Snap.Search` - for performing searches\n\n* `Snap.Multi` - for performing multiple searches in a single query\n\n* `Snap.Bulk` - a convenience wrapper around bulk operations, using `Stream`\n  to stream actions (such as `Snap.Bulk.Action.Create`) to be performed\n  against the `Snap.Cluster`.\n\nAdditionally, there are other supporting modules:\n\n* `Snap.HTTPClient` - defines how to send HTTP requests.\n  Comes with a built in adapter for `Finch` (`Snap.HTTPClient.Adapters.Finch`)\n\n* `Snap.Auth` - defines how an HTTP request is modified to include\n  authentication headers. `Snap.Auth.Plain` implements HTTP Basic Auth.\n\n* `Snap.Cluster.Namespace` - for isolating cluster and processes to work on\n  separate indexes","ref":"Snap.html","title":"Snap","type":"module"},{"doc":"`Snap.Cluster` is a wrapped around an Elasticsearch cluster. We can define\nit like so:\n\n```\ndefmodule MyApp.Cluster do\n  use Snap.Cluster, otp_app: :my_app\nend\n```\n\nThe configuration for the cluster can be defined in your config:\n\n```\nconfig :my_app, MyApp.Cluster,\n  url: \"http://localhost:9200\",\n  username: \"username\",\n  password: \"password\"\n```\n\nOr you can load it dynamically by implementing `c:Snap.Cluster.init/1`.\n\nEach cluster defines `start_link/1` which must be invoked before using the\ncluster and optionally accepts an explicit config. It creates the\nsupervision tree, including the connection pool.\n\nInclude it in your application:\n\n```\ndef start(_type, _args) do\n  children = [\n    {MyApp.Cluster, []}\n  ]\n\n  opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```","ref":"Snap.html#module-set-up","title":"Set up - Snap","type":"module"},{"doc":"The following configuration options are supported:\n\n* `url` - the URL of the Elasticsearch HTTP endpoint (required)\n* `username` - the username used to access the cluster\n* `password` - the password used to access the cluster\n* `auth` - the auth module used to configure the HTTP authentication headers\n  (defaults to `Snap.Auth.Plain`)\n* `http_client_adapter` - the adapter that will be used to send HTTP requests.\n  Check `Snap.HTTPClient` for more information.\n  (defaults to `Snap.HTTPClient.Adapters.Finch`)\n* `telemetry_prefix` - the prefix of the telemetry events (defaults to\n  `[:my_app, :snap]`)\n* `index_namespace` - see `Snap.Cluster.Namespace` for details (defaults to\n  `nil`)","ref":"Snap.html#module-config","title":"Config - Snap","type":"module"},{"doc":"Snap supports sending `Telemetry` events on each HTTP request. It sends one\nevent per query, of the name `[:my_app, :snap, :request]`.\n\nThe telemetry event has the following measurements:\n\n* `response_time` - how long the request took to return\n* `decode_time` - how long the response took to decode into a map or\n  exception\n* `total_time` - how long everything took in total\n\nIn addition, the metadata contains a map of:\n\n* `method` - the HTTP method used\n* `path` - the path requested\n* `port` - the port requested\n* `host` - the host requested\n* `headers` - a list of the headers sent\n* `body` - the body sent\n* `result` - the result returned to the user","ref":"Snap.html#module-telemetry","title":"Telemetry - Snap","type":"module"},{"doc":"See `Snap.Test` for details.","ref":"Snap.html#module-testing","title":"Testing - Snap","type":"module"},{"doc":"Supports streaming bulk operations against a `Snap.Cluster`.","ref":"Snap.Bulk.html","title":"Snap.Bulk","type":"module"},{"doc":"Performs a bulk operation.\n\nTakes an `Enumerable` of action structs, where each struct is one of:\n\n* `Snap.Bulk.Action.Create`\n* `Snap.Bulk.Action.Index`\n* `Snap.Bulk.Action.Update`\n* `Snap.Bulk.Action.Delete`\n\n```\nactions = [\n  %Snap.Bulk.Action.Create{_id: 1, doc: %{foo: \"bar\"}},\n  %Snap.Bulk.Action.Create{_id: 2, doc: %{foo: \"bar\"}},\n  %Snap.Bulk.Action.Create{_id: 3, doc: %{foo: \"bar\"}}\n]\n\nactions\n|> Snap.Bulk.perform(Cluster, \"index\")\n```\n\nIt chunks the `Enumerable` into pages, and pauses between pages for\nElasticsearch to catch up. Uses `Stream` under the hood, so you can lazily\nfeed it a stream of actions, such as out of an `Ecto.Repo` to bulk load\ndocuments from an SQL database.\n\nIf no errors occur on any page it returns `:ok`. If any errors occur, on\nany page, it returns `{:error, %Snap.BulkError{}}`, containing a list of\nthe errors. It will continue to the end of the stream, even if errors\noccur.\n\nOptions:\n\n* `page_size` - defines the size of each page, defaulting to 5000 actions.\n* `page_wait` - defines wait period between pages in ms, defaulting to\n  15000ms.\n* `max_errors` - aborts when the number of errors returned exceedes this\n  count (defaults to `nil`, which will run to the end)\n* `request_opts` - defines the options to be used with `Snap.Request`\n\nAny other options, such as `pipeline: \"foo\"` are passed through as query\nparameters to the [Bulk\nAPI](https://www.elastic.co/guide/en/elasticsearch/reference/current/docs-bulk.html)\nendpoint.","ref":"Snap.Bulk.html#perform/4","title":"Snap.Bulk.perform/4","type":"function"},{"doc":"Defines a cluster.\n\nA cluster maps to an Elasticsearch endpoint.\n\nWhen used, the cluster expects `:otp_app` as an option. The `:otp_app`\nshould point to an OTP application that has the cluster configuration. For\nexample, this cluster:\n\n```\ndefmodule MyApp.Cluster do\n  use Snap.Cluster, otp_app: :my_app\nend\n```\n\nCan be configured with:\n\n```\nconfig :my_app, MyApp.Cluster,\n  url: \"http://localhost:9200\",\n  username: \"username\",\n  password: \"password\",\n  index_namespace: \"app-dev\"\n```\n\nFor details about how index namespacing works, see `Snap.Cluster.Namespace`.","ref":"Snap.Cluster.html","title":"Snap.Cluster","type":"behaviour"},{"doc":"Returns the config in use by this cluster.","ref":"Snap.Cluster.html#c:config/0","title":"Snap.Cluster.config/0","type":"callback"},{"doc":"Sends a DELETE request.\n\nReturns either:\n\n* `{:ok, response}` - where response is a map representing the parsed JSON response.\n* `{:error, error}` - where the error can be a struct of either:\n  * `Snap.ResponseError`\n  * `Snap.HTTPClient.Error`\n  * `Jason.DecodeError`","ref":"Snap.Cluster.html#c:delete/4","title":"Snap.Cluster.delete/4","type":"callback"},{"doc":"Sends a GET request.\n\nReturns either:\n\n* `{:ok, response}` - where response is a map representing the parsed JSON response.\n* `{:error, error}` - where the error can be a struct of either:\n  * `Snap.ResponseError`\n  * `Snap.HTTPClient.Error`\n  * `Jason.DecodeError`","ref":"Snap.Cluster.html#c:get/4","title":"Snap.Cluster.get/4","type":"callback"},{"doc":"Sets up the config for the cluster.\n\nOverride this to dynamically load a config from somewhere other than your\napplication config.","ref":"Snap.Cluster.html#c:init/1","title":"Snap.Cluster.init/1","type":"callback"},{"doc":"Sends a POST request.\n\nReturns either:\n\n* `{:ok, response}` - where response is a map representing the parsed JSON response.\n* `{:error, error}` - where the error can be a struct of either:\n  * `Snap.ResponseError`\n  * `Snap.HTTPClient.Error`\n  * `Jason.DecodeError`","ref":"Snap.Cluster.html#c:post/5","title":"Snap.Cluster.post/5","type":"callback"},{"doc":"Sends a PUT request.\n\nReturns either:\n\n* `{:ok, response}` - where response is a map representing the parsed JSON response.\n* `{:error, error}` - where the error can be a struct of either:\n  * `Snap.ResponseError`\n  * `Snap.HTTPClient.Error`\n  * `Jason.DecodeError`","ref":"Snap.Cluster.html#c:put/5","title":"Snap.Cluster.put/5","type":"callback"},{"doc":"The body of the HTTP request","ref":"Snap.Cluster.html#t:body/0","title":"Snap.Cluster.body/0","type":"type"},{"doc":"Options available for configuring the Cluster","ref":"Snap.Cluster.html#t:config_opts/0","title":"Snap.Cluster.config_opts/0","type":"type"},{"doc":"An error from an HTTP operation","ref":"Snap.Cluster.html#t:error/0","title":"Snap.Cluster.error/0","type":"type"},{"doc":"Any additional HTTP headers sent with the request","ref":"Snap.Cluster.html#t:headers/0","title":"Snap.Cluster.headers/0","type":"type"},{"doc":"Options passed through to the request","ref":"Snap.Cluster.html#t:opts/0","title":"Snap.Cluster.opts/0","type":"type"},{"doc":"The query params, which will be appended to the path","ref":"Snap.Cluster.html#t:params/0","title":"Snap.Cluster.params/0","type":"type"},{"doc":"The path of the HTTP endpoint","ref":"Snap.Cluster.html#t:path/0","title":"Snap.Cluster.path/0","type":"type"},{"doc":"The result from an HTTP operation","ref":"Snap.Cluster.html#t:result/0","title":"Snap.Cluster.result/0","type":"type"},{"doc":"A successful results from an HTTP operation","ref":"Snap.Cluster.html#t:success/0","title":"Snap.Cluster.success/0","type":"type"},{"doc":"Manages the namespacing of a cluster, so a cluster can be scoped to only\noperate on indexes with a specific prefix.\n\nThis is useful for a few purposes:\n\n- Running multiple instances of `Snap`, across completely different\n  applications, against a single cluster. You might not want do this in\n  production, but it's useful to only run one copy of ElasticSearch/OpenSearch\n  locally when working on multiple applications.\n\n- Running multiple environments of the same application. You don't want your\n  tests to blow away all your development data each time they run.\n\n- Running tests in parallel, so each test process cannot see or act upon\n  indexes that another test is operating on. This gives you the benefits of\n  `Ecto` style sandboxing, meaning your tests can run with `async: true` even\n  though ElasticSearch/OpenSearch doesn't support isolated transactions.\n\nAll of the convenience modules respect namespacing:\n\n* `Snap.Bulk`\n* `Snap.Document`\n* `Snap.Indexes`\n* `Snap.Multi`\n* `Snap.Search`\n\nThe low level HTTP API on `Snap` does not.\n\nThere are two types of namespace:","ref":"Snap.Cluster.Namespace.html","title":"Snap.Cluster.Namespace","type":"module"},{"doc":"You can namespace the entire `Snap.Cluster` in the config. This is used to\nachieve isolation at a per application and environment level.\n\nFor example, to isolate your development and test environments, set\n`index_namespace: \"dev\"` in your dev config, and `index_namespace: \"test\"` in\nyour test config.\n\nYou might even want to isolate multiple applications on a per environment\nbasis, with `index_namespace: \"app1-dev\"`, for example.","ref":"Snap.Cluster.Namespace.html#module-config-namespace","title":"Config namespace - Snap.Cluster.Namespace","type":"module"},{"doc":"The process namespace is used to achieve isolation between parallel running\ntests. It's not likely you'll want to use this in other situations.\n\nFor more information, see `Snap.Test`.","ref":"Snap.Cluster.Namespace.html#module-process-namespace","title":"Process namespace - Snap.Cluster.Namespace","type":"module"},{"doc":"Given an index, adds the namespace to the supplied index for the\n`Snap.Cluster` in the currently running process.","ref":"Snap.Cluster.Namespace.html#add_namespace_to_index/2","title":"Snap.Cluster.Namespace.add_namespace_to_index/2","type":"function"},{"doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Snap.Cluster.Namespace.html#child_spec/1","title":"Snap.Cluster.Namespace.child_spec/1","type":"function"},{"doc":"Clears the previously set process namespace, if any.","ref":"Snap.Cluster.Namespace.html#clear_process_namespace/2","title":"Snap.Cluster.Namespace.clear_process_namespace/2","type":"function"},{"doc":"Returns the previously set process namespace, if any.","ref":"Snap.Cluster.Namespace.html#get_process_namespace/2","title":"Snap.Cluster.Namespace.get_process_namespace/2","type":"function"},{"doc":"Returns a boolean indicating whether the namespaced index is inside the currently\ndefined namespace.","ref":"Snap.Cluster.Namespace.html#index_in_namespace?/2","title":"Snap.Cluster.Namespace.index_in_namespace?/2","type":"function"},{"doc":"Given an index, returns the index namespace for the `Snap.Cluster` in the\ncurrently running process.","ref":"Snap.Cluster.Namespace.html#index_namespace/1","title":"Snap.Cluster.Namespace.index_namespace/1","type":"function"},{"doc":"Same as `set_process_namespace/3`, but for the running process.","ref":"Snap.Cluster.Namespace.html#set_process_namespace/2","title":"Snap.Cluster.Namespace.set_process_namespace/2","type":"function"},{"doc":"Set the index namespace of the provided `pid`. Subsequent operations through\nconvenience APIs are performed on namespaced indexes.","ref":"Snap.Cluster.Namespace.html#set_process_namespace/3","title":"Snap.Cluster.Namespace.set_process_namespace/3","type":"function"},{"doc":"Remove the namespace prefix from a namespaced index, returning the remainder.","ref":"Snap.Cluster.Namespace.html#strip_namespace/2","title":"Snap.Cluster.Namespace.strip_namespace/2","type":"function"},{"doc":"Convenience API into the [Document API](https://www.elastic.co/guide/en/elasticsearch/reference/current/docs.html).","ref":"Snap.Document.html","title":"Snap.Document","type":"module"},{"doc":"Creates a new document in the index. The ID will be assigned automatically.","ref":"Snap.Document.html#add/5","title":"Snap.Document.add/5","type":"function"},{"doc":"Creates a document in the index with the specified ID. Fails if a document already exists at that ID.","ref":"Snap.Document.html#create/6","title":"Snap.Document.create/6","type":"function"},{"doc":"Deletes a document in the index at the specified ID.","ref":"Snap.Document.html#delete/5","title":"Snap.Document.delete/5","type":"function"},{"doc":"Gets a document in the index with the specified ID","ref":"Snap.Document.html#get/5","title":"Snap.Document.get/5","type":"function"},{"doc":"Creates or updates a document in the index with the specified ID. Overwrite it if it already exists.","ref":"Snap.Document.html#index/6","title":"Snap.Document.index/6","type":"function"},{"doc":"Updates the document at the ID. See the ElasticSearch/OpenSearch docs for more\ndetails about [how updates are\nperformed](https://www.elastic.co/guide/en/elasticsearch/reference/current/docs-update.html).","ref":"Snap.Document.html#update/6","title":"Snap.Document.update/6","type":"function"},{"doc":"Helper functions around index management.","ref":"Snap.Indexes.html","title":"Snap.Indexes","type":"module"},{"doc":"Creates an alias for a versioned index, removing any existing aliases.","ref":"Snap.Indexes.html#alias/4","title":"Snap.Indexes.alias/4","type":"function"},{"doc":"Deletes older timestamped indexes.","ref":"Snap.Indexes.html#cleanup/4","title":"Snap.Indexes.cleanup/4","type":"function"},{"doc":"Creates an index.","ref":"Snap.Indexes.html#create/4","title":"Snap.Indexes.create/4","type":"function"},{"doc":"Deletes an index.","ref":"Snap.Indexes.html#delete/3","title":"Snap.Indexes.delete/3","type":"function"},{"doc":"Creates and loads a new index, switching the alias to it with zero-downtime.\n\nTakes an `Enumerable` of `Snap.Bulk` actions, and builds a new index from\nit. Refreshes it, updates the alias to it, and cleans up the old indexes,\nleaving the previous one behind.\n\nMay return `t:Snap.Cluster.error/0` or a `Snap.BulkError` containing a list\nof failed bulk actions.","ref":"Snap.Indexes.html#hotswap/5","title":"Snap.Indexes.hotswap/5","type":"function"},{"doc":"Lists all the indexes in the cluster.","ref":"Snap.Indexes.html#list/2","title":"Snap.Indexes.list/2","type":"function"},{"doc":"Lists all the timestamp versioned indexes starting with the prefix.","ref":"Snap.Indexes.html#list_starting_with/3","title":"Snap.Indexes.list_starting_with/3","type":"function"},{"doc":"Refreshes an index.","ref":"Snap.Indexes.html#refresh/3","title":"Snap.Indexes.refresh/3","type":"function"},{"doc":"Performs searches against an ElasticSearch cluster.","ref":"Snap.Search.html","title":"Snap.Search","type":"module"},{"doc":"Runs a count of the documents in an index, using an optional query.","ref":"Snap.Search.html#count/6","title":"Snap.Search.count/6","type":"function"},{"doc":"Makes a search against an ElasticSearch cluster and parses the response.\n\nPerforms a search against an index using a POST request, and parses the\nresponse into a `Snap.SearchResponse`.\n\n`Snap.SearchResponse` implements `Enumerable`, so you can count and iterate\ndirectly on the struct.","ref":"Snap.Search.html#search/6","title":"Snap.Search.search/6","type":"function"},{"doc":"query = %{query: %{match_all: %{}}}\n    {:ok, response} = Snap.Search.search(Cluster, \"index\", query)\n\n    IO.inspect(response.took)\n    Enum.each(response, fn hit -> IO.inspect(hit.score) end)","ref":"Snap.Search.html#search/6-examples","title":"Examples - Snap.Search.search/6","type":"function"},{"doc":"Helpers around testing with `Snap`.\n\nUnlike SQL databases, ElasticSearch/OpenSearch does not provide transaction\nisolation, which is how `Ecto.Adapters.SQL.Sandbox` isolates test\nprocesses, allowing multiple tests to run asynchronously.\n\n`Snap` uses a different approach. It can set an index namespace for a running\nprocess, which scopes all the convenience API operations to that index\nnamespace, so they don't collide with each other.\n\nThis allows tests to run asynchronously, improving the performance.\n\nTo set this up, you'll want to use [tags](https://hexdocs.pm/ex_unit/1.12.3/ExUnit.Case.html#module-tags) in `ExUnit`\n\nIn your test you'll do something similar to:\n\n```\nsetup context do\n  if context[:snap] do\n    namespace = Snap.Test.generate_namespace_for_pid(self())\n    Snap.Cluster.Namespace.set_process_namespace(Cluster, namespace)\n    Snap.Test.drop_indexes(Cluster)\n\n    on_exit(fn ->\n      Snap.Cluster.Namespace.set_process_namespace(Cluster, namespace)\n      Snap.Test.drop_indexes(Cluster)\n    end)\n  end\nend\n\n@tag :snap\ntest \"test something with snap\" do\n  ...\nend\n```\n\nThis generate a unique namespace for the running process and assigns to the\ncluster. It clears the indexes to make sure the test is starting from scratch.\n\nIt setups up a `on_exit/1` callback, which runs in a separate process after\nthe test has completed. The namespace is passed through from the test's\nprocess, so it can teardown all the indexes created by the test.","ref":"Snap.Test.html","title":"Snap.Test","type":"module"},{"doc":"Drops all the indexes on the cluster. Beware!","ref":"Snap.Test.html#drop_indexes/1","title":"Snap.Test.drop_indexes/1","type":"function"},{"doc":"Generates a `String` namespace for the provided pid, by hashing it.","ref":"Snap.Test.html#generate_namespace_for_pid/1","title":"Snap.Test.generate_namespace_for_pid/1","type":"function"},{"doc":"Takes the process namespace from one process and set it on another. Useful if\nyour tests start multiple processes and they all need to have the same view of\nyour cluster.","ref":"Snap.Test.html#propagate_namespace/3","title":"Snap.Test.propagate_namespace/3","type":"function"},{"doc":"Defines how HTTP request is transformed to add authentication.","ref":"Snap.Auth.html","title":"Snap.Auth","type":"behaviour"},{"doc":"Modifies an HTTP request to include authentication details","ref":"Snap.Auth.html#c:sign/5","title":"Snap.Auth.sign/5","type":"callback"},{"doc":"","ref":"Snap.Auth.html#t:response/0","title":"Snap.Auth.response/0","type":"type"},{"doc":"","ref":"Snap.Auth.html#t:t/0","title":"Snap.Auth.t/0","type":"type"},{"doc":"Implements HTTP Basic Auth, if necessary.\n\nIf the cluster config defines a username and password, an `Authorization:\nBasic` header is added to the request.\n\n```\nconfig :my_app, MyApp.Cluster,\n  url: \"http://localhost:9200\",\n  username: \"foo\",\n  password: \"bar\n```\n\nOr you can define a username and password in the URL itself:\n\n```\nconfig :my_app, MyApp.Cluster,\n  url: \"http://username:password@localhost:9200\"\n```\n\nIf no username or password is defined, no `Authorization` header is added,\nmaking it suitable for local development.","ref":"Snap.Auth.Plain.html","title":"Snap.Auth.Plain","type":"module"},{"doc":"","ref":"Snap.Auth.Plain.html#sign/5","title":"Snap.Auth.Plain.sign/5","type":"function"},{"doc":"Behaviour for the HTTP client used by the `Snap.Cluster`.\n\nBy default, it uses the `Snap.HTTPClient.Adapters.Finch` for making requests.\n\nYou can configure the Cluster with your own adapter:\n\n```\nconfig :my_app, MyApp.Cluster,\n  http_client_adapter: MyHTTPClientAdapter\n```\n\nThe adapter can be configured passing a tuple:\n\n```\nconfig :my_app, MyApp.Cluster,\n  http_client_adapter: {MyHTTPClientAdapter, some_config_for_adapter: \"config_value\"}\n```","ref":"Snap.HTTPClient.html","title":"Snap.HTTPClient","type":"behaviour"},{"doc":"","ref":"Snap.HTTPClient.html#child_spec/1","title":"Snap.HTTPClient.child_spec/1","type":"function"},{"doc":"Returns a specification to start this module under the `Snap` supervisor tree.\n\nIf the adapter doesn't need to start in the supervisor tree, you can return `:skip`.","ref":"Snap.HTTPClient.html#c:child_spec/1","title":"Snap.HTTPClient.child_spec/1","type":"callback"},{"doc":"","ref":"Snap.HTTPClient.html#request/6","title":"Snap.HTTPClient.request/6","type":"function"},{"doc":"Executes the request with the configured adapter.","ref":"Snap.HTTPClient.html#c:request/6","title":"Snap.HTTPClient.request/6","type":"callback"},{"doc":"","ref":"Snap.HTTPClient.html#t:body/0","title":"Snap.HTTPClient.body/0","type":"type"},{"doc":"","ref":"Snap.HTTPClient.html#t:child_spec/0","title":"Snap.HTTPClient.child_spec/0","type":"type"},{"doc":"","ref":"Snap.HTTPClient.html#t:headers/0","title":"Snap.HTTPClient.headers/0","type":"type"},{"doc":"","ref":"Snap.HTTPClient.html#t:method/0","title":"Snap.HTTPClient.method/0","type":"type"},{"doc":"","ref":"Snap.HTTPClient.html#t:t/0","title":"Snap.HTTPClient.t/0","type":"type"},{"doc":"","ref":"Snap.HTTPClient.html#t:url/0","title":"Snap.HTTPClient.url/0","type":"type"},{"doc":"Built in adapter using `Finch`.\n\nYou can also configure this adapter by explicitly setting the `http_client_adapter`\nin the `Snap.Cluster` configuration with a tuple `{Snap.HTTPClient.Adapters.Finch, config}`.\nFor example:\n\n```\nconfig :my_app, MyApp.Cluster,\n  http_client_adapter: {Snap.HTTPClient.Adapters.Finch, pool_size: 20}\n```\n\nYou can check the `t:config/0` for docs about the available configurations.","ref":"Snap.HTTPClient.Adapters.Finch.html","title":"Snap.HTTPClient.Adapters.Finch","type":"module"},{"doc":"Available options for configuring the Finch adapter. For more information about the options,\nyou can check [Finch's official docs](https://hexdocs.pm/finch/Finch.html#start_link/1-pool-configuration-options).\n\n  * `pool_size`: Set the pool size. Defaults to `5`.\n  * `conn_opts`: Connection options passed to `Mint.HTTP.connect/4`. Defaults to `[]`.\n  * `accept_encoding`: The default 'Accept-Encoding' header to send as a\n    string. Defaults to `gzip`, so the server will return gzip compressed\n    responses if configured correctly. Set to `false` to disable.","ref":"Snap.HTTPClient.Adapters.Finch.html#t:config/0","title":"Snap.HTTPClient.Adapters.Finch.config/0","type":"type"},{"doc":"Response of a http request.","ref":"Snap.HTTPClient.Response.html","title":"Snap.HTTPClient.Response","type":"module"},{"doc":"","ref":"Snap.HTTPClient.Response.html#t:status/0","title":"Snap.HTTPClient.Response.status/0","type":"type"},{"doc":"","ref":"Snap.HTTPClient.Response.html#t:t/0","title":"Snap.HTTPClient.Response.t/0","type":"type"},{"doc":"Represents a create step in a `Snap.Bulk` operation","ref":"Snap.Bulk.Action.Create.html","title":"Snap.Bulk.Action.Create","type":"module"},{"doc":"","ref":"Snap.Bulk.Action.Create.html#t:t/0","title":"Snap.Bulk.Action.Create.t/0","type":"type"},{"doc":"Represents a delete step in a `Snap.Bulk` operation","ref":"Snap.Bulk.Action.Delete.html","title":"Snap.Bulk.Action.Delete","type":"module"},{"doc":"","ref":"Snap.Bulk.Action.Delete.html#t:t/0","title":"Snap.Bulk.Action.Delete.t/0","type":"type"},{"doc":"Represents an index step in a `Snap.Bulk` operation","ref":"Snap.Bulk.Action.Index.html","title":"Snap.Bulk.Action.Index","type":"module"},{"doc":"","ref":"Snap.Bulk.Action.Index.html#t:t/0","title":"Snap.Bulk.Action.Index.t/0","type":"type"},{"doc":"Represents an update step in a `Snap.Bulk` operation","ref":"Snap.Bulk.Action.Update.html","title":"Snap.Bulk.Action.Update","type":"module"},{"doc":"","ref":"Snap.Bulk.Action.Update.html#t:t/0","title":"Snap.Bulk.Action.Update.t/0","type":"type"},{"doc":"Provides a high level abstraction over the [Multi Search\nAPI](https://www.elastic.co/guide/en/elasticsearch/reference/current/search-multi-search.html)\nwhich allows the user to perform multiple searches in a single request.\n\nExample usage:\n\n    Multi.new()\n    |> Multi.add(query_1, id: \"query-1\")\n    |> Multi.add(query_2, id: \"query-2\")\n    |> Multi.run(Cluster, index)\n\nThis returns a `Snap.Multi.Response`, with a `searches` field containing a\nlist of responses.\n\n    {:ok, %Snap.Multi.Response{searches: [\n      {\"query-1\", %Snap.SearchResponse{...}},\n      {\"query-2\", %Snap.SearchResponse{...}}\n    ]}}\n\nEach query can be named, using any value you like, by passing an `id: \"foo\"`\ninto `Multi.add`. The list in `Snap.Multi.Response` contains tuple pairs where\nthe first value is the ID and the second is the result of the query.\n\nIf you choose not to name a query the ID in the tuple will be `nil`.\n\nIf your query IDs are unique you can convert this to a `Map` for easy lookup\nlater using `Enum.into(response.searches, %{})`.","ref":"Snap.Multi.html","title":"Snap.Multi","type":"module"},{"doc":"Append to a `Snap.Multi` request. The `body` is required. If you pass an `id`\ninto the headers, this will be used to name the query in the responses list\nand won't be passed through as a header in the request.","ref":"Snap.Multi.html#add/3","title":"Snap.Multi.add/3","type":"function"},{"doc":"Build a `Snap.Multi` request.","ref":"Snap.Multi.html#new/0","title":"Snap.Multi.new/0","type":"function"},{"doc":"Perform the `Snap.Multi` request. This returns `{:ok, Snap.Multi.Response}` or\nan error.","ref":"Snap.Multi.html#run/6","title":"Snap.Multi.run/6","type":"function"},{"doc":"","ref":"Snap.Multi.html#t:t/0","title":"Snap.Multi.t/0","type":"type"},{"doc":"Represents a successful response for a `Snap.Multi` request.","ref":"Snap.Multi.Response.html","title":"Snap.Multi.Response","type":"module"},{"doc":"","ref":"Snap.Multi.Response.html#t:t/0","title":"Snap.Multi.Response.t/0","type":"type"},{"doc":"Represents an individual aggregation dictionary from a\n[Search Aggregation API](https://www.elastic.co/guide/en/elasticsearch/reference/current/search-aggregations.html)\nresponse.","ref":"Snap.Aggregation.html","title":"Snap.Aggregation","type":"module"},{"doc":"","ref":"Snap.Aggregation.html#new/1","title":"Snap.Aggregation.new/1","type":"function"},{"doc":"","ref":"Snap.Aggregation.html#t:t/0","title":"Snap.Aggregation.t/0","type":"type"},{"doc":"Represents an individual hit dictionary from a [Search API](https://www.elastic.co/guide/en/elasticsearch/reference/current/search.html) response.","ref":"Snap.Hit.html","title":"Snap.Hit","type":"module"},{"doc":"","ref":"Snap.Hit.html#new/1","title":"Snap.Hit.new/1","type":"function"},{"doc":"","ref":"Snap.Hit.html#t:t/0","title":"Snap.Hit.t/0","type":"type"},{"doc":"Represents the `hits` dictionary returned from an ElasticSearch [Search API](https://www.elastic.co/guide/en/elasticsearch/reference/current/search.html) response.\n\nImplements `Enumerable`, so you can iterate directly over the struct.","ref":"Snap.Hits.html","title":"Snap.Hits","type":"module"},{"doc":"","ref":"Snap.Hits.html#new/1","title":"Snap.Hits.new/1","type":"function"},{"doc":"","ref":"Snap.Hits.html#t:t/0","title":"Snap.Hits.t/0","type":"type"},{"doc":"Represents the response from ElasticSearch's [Search API](https://www.elastic.co/guide/en/elasticsearch/reference/current/search.html).\n\nImplements `Enumerable`, so you can iterate directly over the struct.","ref":"Snap.SearchResponse.html","title":"Snap.SearchResponse","type":"module"},{"doc":"","ref":"Snap.SearchResponse.html#build_aggregations/1","title":"Snap.SearchResponse.build_aggregations/1","type":"function"},{"doc":"","ref":"Snap.SearchResponse.html#new/1","title":"Snap.SearchResponse.new/1","type":"function"},{"doc":"","ref":"Snap.SearchResponse.html#t:t/0","title":"Snap.SearchResponse.t/0","type":"type"},{"doc":"Represents an individual suggest dictionary from a [Search API](https://www.elastic.co/guide/en/elasticsearch/reference/current/search.html) response.","ref":"Snap.Suggest.html","title":"Snap.Suggest","type":"module"},{"doc":"","ref":"Snap.Suggest.html#new/1","title":"Snap.Suggest.new/1","type":"function"},{"doc":"","ref":"Snap.Suggest.html#t:t/0","title":"Snap.Suggest.t/0","type":"type"},{"doc":"Represents an individual `suggest` / `options` dictionary from a [Search API](https://www.elastic.co/guide/en/elasticsearch/reference/current/search.html) response.","ref":"Snap.Suggest.Option.html","title":"Snap.Suggest.Option","type":"module"},{"doc":"","ref":"Snap.Suggest.Option.html#new/1","title":"Snap.Suggest.Option.new/1","type":"function"},{"doc":"","ref":"Snap.Suggest.Option.html#t:t/0","title":"Snap.Suggest.Option.t/0","type":"type"},{"doc":"Represents the `suggest` / `options` dictionary returned from an ElasticSearch [Search API](https://www.elastic.co/guide/en/elasticsearch/reference/current/search.html) response.","ref":"Snap.Suggest.Options.html","title":"Snap.Suggest.Options","type":"module"},{"doc":"","ref":"Snap.Suggest.Options.html#new/1","title":"Snap.Suggest.Options.new/1","type":"function"},{"doc":"","ref":"Snap.Suggest.Options.html#t:t/0","title":"Snap.Suggest.Options.t/0","type":"type"},{"doc":"Represents the `suggest` dictionary returned from an ElasticSearch [Search API](https://www.elastic.co/guide/en/elasticsearch/reference/current/search.html) response.","ref":"Snap.Suggests.html","title":"Snap.Suggests","type":"module"},{"doc":"","ref":"Snap.Suggests.html#new/1","title":"Snap.Suggests.new/1","type":"function"},{"doc":"","ref":"Snap.Suggests.html#t:t/0","title":"Snap.Suggests.t/0","type":"type"},{"doc":"Represents a list of errors, collected from a `Snap.Bulk` operation.","ref":"Snap.BulkError.html","title":"Snap.BulkError","type":"exception"},{"doc":"","ref":"Snap.BulkError.html#t:t/0","title":"Snap.BulkError.t/0","type":"type"},{"doc":"Represents an Error returned by the HTTPClient.\n\nUsually this wraps an error coming from the adapter that implements the `Snap.HTTPClient` behaviour,\nsuch as `Mint.HTTPError`.","ref":"Snap.HTTPClient.Error.html","title":"Snap.HTTPClient.Error","type":"exception"},{"doc":"* `:reason` - the error reason represented by an atom.\n  * `:origin` - the origin of the error. Contains the original error coming from the adapter of the\n  `Snap.HTTPClient` behaviour, such as `Mint.HTTPError` or `Mint.TransportError`.","ref":"Snap.HTTPClient.Error.html#module-struct-fields","title":"Struct Fields - Snap.HTTPClient.Error","type":"exception"},{"doc":"","ref":"Snap.HTTPClient.Error.html#new/2","title":"Snap.HTTPClient.Error.new/2","type":"function"},{"doc":"","ref":"Snap.HTTPClient.Error.html#unknown/1","title":"Snap.HTTPClient.Error.unknown/1","type":"function"},{"doc":"","ref":"Snap.HTTPClient.Error.html#t:t/0","title":"Snap.HTTPClient.Error.t/0","type":"type"},{"doc":"Represents an HTTP error returned while executing a query.","ref":"Snap.HTTPError.html","title":"Snap.HTTPError","type":"exception"},{"doc":"","ref":"Snap.HTTPError.html#exception_from_response/1","title":"Snap.HTTPError.exception_from_response/1","type":"function"},{"doc":"","ref":"Snap.HTTPError.html#message/1","title":"Snap.HTTPError.message/1","type":"function"},{"doc":"Represents an Elasticsearch exception raised while executing a query.","ref":"Snap.ResponseError.html","title":"Snap.ResponseError","type":"exception"},{"doc":"","ref":"Snap.ResponseError.html#exception_from_json/1","title":"Snap.ResponseError.exception_from_json/1","type":"function"},{"doc":"","ref":"Snap.ResponseError.html#t:t/0","title":"Snap.ResponseError.t/0","type":"type"},{"doc":"# Snap\n\n[![Hex pm](http://img.shields.io/hexpm/v/snap.svg?style=flat)](https://hex.pm/packages/snap)\n\nSnap is an Elasticsearch client. It provides a flexible, performant API on\ntop of your Elasticsearch cluster, supporting high level features like\nversioned index management, while also providing a convenient interface into\nlow level operations.\n\nSee the full [API docs](https://hexdocs.pm/snap).","ref":"readme.html","title":"Snap","type":"extras"},{"doc":"- Versioned index management with zero-downtime hotswapping (compatible with [`elasticsearch`](https://github.com/danielberkompas/elasticsearch-elixir))\n- Streaming bulk operations\n- Connection pooling\n- Telemetry events\n- High level interface over the [Multi Search API](https://www.elastic.co/guide/en/elasticsearch/reference/current/search-multi-search.html)","ref":"readme.html#features","title":"Features - Snap","type":"extras"},{"doc":"The package can be installed by adding `snap` to your list of dependencies in\n`mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:snap, \"~> 0.9\"},\n    {:finch, \"~> 0.16\"}, # By default, Snap uses Finch to make HTTP requests\n  ]\nend\n```\n\nSnap supports Elixir 1.12 or later.","ref":"readme.html#installation","title":"Installation - Snap","type":"extras"},{"doc":"Implement your own cluster module, similar to an `Ecto.Repo`:\n\n```elixir\ndefmodule MyApp.Cluster do\n  use Snap.Cluster, otp_app: :my_app\nend\n```\n\nConfigure it:\n\n```elixir\nconfig :my_app, MyApp.Cluster,\n  url: \"http://localhost:9200\",\n  username: \"my_username\",\n  password: \"my_password\"\n```\n\nThen wire it into your application supervisor:\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    {MyApp.Cluster, []}\n  ]\n\n  opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```\n\nNow you can perform operations on your cluster:\n\n```elixir\n{:ok, %{\"count\" => count}} = MyApp.Cluster.get(\"/my-index/_count\")\n```","ref":"readme.html#usage","title":"Usage - Snap","type":"extras"},{"doc":"If you want to test your app that uses this library, but don't want to have integration tests\nwith a Elasticsearch instance running in you local dev environment,\nyou can mock the responses using a custom HTTP client adapter.\n\nSupposing you are using [mox](https://github.com/dashbitco/mox), you can do something like this:\n\n```elixir\n# in test_helper.exs\nMox.defmock(HTTPClientMock, for: Snap.HTTPClient)\nMox.stub(HTTPClientMock, :child_spec, fn _config -> :skip end)\n\n# in config/test.exs\nconfig :my_app, MyApp.Cluster, http_client_adapter: HTTPClientMock\n\n# in a test file\nMox.expect(HTTPClientMock, :request, fn _cluster, _method, _url, _headers, _body, _opts\n  body = \"{}\" # valid json\n  {:ok, %Snap.HTTPClient.Response{status: 200, headers: [], body: body}}\nend)\n```\n\nSee the [API documentation](https://hexdocs.pm/snap) for more advanced features.","ref":"readme.html#testing","title":"Testing - Snap","type":"extras"},{"doc":"# Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"- Enabled gzip compression in default HTTP adapter using `Accept-Heading` header, which speeds up large response objects","ref":"changelog.html#0-10-0","title":"0.10.0 - Changelog","type":"extras"},{"doc":"- Added `Snap.Document` for interacting with individual documents\n- Added `count/3` to `Snap.Search` for performing counts with or without a query\n- Added support for index namespacing to isolate different Snap application, environments and even process from seeing each other's indexes - see `Snap.Cluster.Namespace` and `Snap.Test` for details","ref":"changelog.html#0-9-0","title":"0.9.0 - Changelog","type":"extras"},{"doc":"- Upgrade `finch`, which allows use of `castore` 1.0\n- Set minimum Elixir version to 1.12","ref":"changelog.html#0-8-1","title":"0.8.1 - Changelog","type":"extras"},{"doc":"- Add a high level interface for the [Multi Search API](https://www.elastic.co/guide/en/elasticsearch/reference/current/search-multi-search.html)","ref":"changelog.html#0-8-0","title":"0.8.0 - Changelog","type":"extras"},{"doc":"- Changed `inner_hits` to be `Snap.Hits` struct to make it consistent with the `hits` at the top level\n- Added support for `suggest` (thanks @maennchen)\n- Relaxed telemetry dependency to allow the use of 0.4 as well as 1.0 (thanks @dershodan)","ref":"changelog.html#0-7-0","title":"0.7.0 - Changelog","type":"extras"},{"doc":"- Added support for modular HTTP client adapters, allowing mock HTTP clients in a test environment (thanks @jotaviobiondo)\n- Added extra fields to `Snap.Aggregation` (thanks @megalithic)","ref":"changelog.html#0-6-0","title":"0.6.0 - Changelog","type":"extras"},{"doc":"- Return `Snap.HTTPError` when ElasticSearch returns an error HTTP status\n  without any accompanying JSON\n- Expose `inner_hits` field on `Snap.Hit` struct","ref":"changelog.html#0-5-2","title":"0.5.2 - Changelog","type":"extras"},{"doc":"- Accept `conn_opts` config options to configure the underlying HTTP transport\n  (thanks @danielxvu)","ref":"changelog.html#0-5-1","title":"0.5.1 - Changelog","type":"extras"},{"doc":"- Update `telemetry` dependency to 1.0","ref":"changelog.html#0-5-0","title":"0.5.0 - Changelog","type":"extras"},{"doc":"- Allow extra options on `Snap.Bulk.perform/4` which are passed into the underlying request\n- Added `matched_queries` and `highlight` fields on the `Snap.Hit` struct.","ref":"changelog.html#0-4-5","title":"0.4.5 - Changelog","type":"extras"},{"doc":"- Added the `host` and `port` to the Telemetry metadata.","ref":"changelog.html#0-4-1","title":"0.4.1 - Changelog","type":"extras"},{"doc":"- Drop the underscore from the response struct keys, as it's just annoying to\n  work with.","ref":"changelog.html#0-4-0","title":"0.4.0 - Changelog","type":"extras"},{"doc":"- Added `Snap.Search` to wrap making searches and parsing the response into\n  structs.","ref":"changelog.html#0-3-0","title":"0.3.0 - Changelog","type":"extras"},{"doc":"- Fixed behaviour of `max_errors` when it's set to `0`.","ref":"changelog.html#0-2-4","title":"0.2.4 - Changelog","type":"extras"},{"doc":"- Added support for an optional `max_errors` parameter in\n  `Snap.Bulk.perform`, which aborts the operation if the number of errors\n  accumulated exceedes this count.","ref":"changelog.html#0-2-3","title":"0.2.3 - Changelog","type":"extras"},{"doc":"- Pass extra opts, such as `pipeline: \"foo\"` in `Snap.Bulk.perform` to the\n  Bulk API endpoint.","ref":"changelog.html#0-2-2","title":"0.2.2 - Changelog","type":"extras"},{"doc":"- Added support in `Snap.Auth.Plain` for defining the username and password in\n  the configured URL.","ref":"changelog.html#0-2-1","title":"0.2.1 - Changelog","type":"extras"},{"doc":"- Introduce `Snap.BulkError` to wrap a list of errors produced from\n  `Snap.Bulk`.","ref":"changelog.html#0-2-0","title":"0.2.0 - Changelog","type":"extras"},{"doc":"- Documentation formatting fix.","ref":"changelog.html#0-1-1","title":"0.1.1 - Changelog","type":"extras"},{"doc":"- First release.","ref":"changelog.html#0-1-0","title":"0.1.0 - Changelog","type":"extras"}],"producer":{"name":"ex_doc","version":[48,46,51,50,46,49]}}